*----------------------------------------------------------------------*
* THE BLACK HOLE                                                       *
* The following list gives details of what is allowed in QDOS speak    *
*                                                                      *
* TRAP #1 - not applicable.                                            *
* TRAP #2 - IO_OPEN & IO_CLOSE                        both work ok.    *
* TRAP #3 - IO_SBYTE, FS_CHECK, FS_FLUSH, FS_POSAB, FS_POSRE           *
*           IO_SSTRG, FS_SAVE                         all work ok.     *
*           IO_PEND, IO_FBYTE, IO_FLINE, IO_FSTRG, IO_EDLIN            *
*           FS_MDINF, FS_HEADR, FS_LOAD               all give ERR_EF. *
*           All other D0 codes give ERR_BP.                            *
*                                                                      *
* Note that IO_SSTRG & FS_SAVE return correct parameters as if they    *
* had done something, ie, A1.L = A1.L + D2.W, D1.W = D2.W              *
*======================================================================*
* Copyright Norman Dunbar                                              *
*           February 1991                                              *
*======================================================================*

Now a list of trap codes that are used in the following source.

err_ef   equ       -10
err_bp   equ       -15

io_sbyte equ       $05
io_sstrg equ       $07
mt_liod  equ       $20
fs_check equ       $40
fs_posre equ       $43
fs_mdinf equ       $45
fs_headr equ       $47
fs_heads equ       $48
fs_save  equ       $49

mm_alchp equ       $C0
mm_rechp equ       $C2
io_name  equ       $122

start      lea.l   drvr_block,a0       linkage block address
           lea.l   dev_io,a1           I/O routine address
           move.l  a1,4(a0)            Into its slot
           lea.l   dev_open,a1         OPEN routine address
           move.l  a1,8(a0)            Into its slot
           lea.l   dev_close,a1        CLOSE routine address
           move.l  a1,$0c(a0)          Into its slot

           moveq   #mt_liod,d0         Now prepare to ...
           trap    #1                  ... link driver into QDOS and
           rts                         exit with any error codes.

drvr_block ds.l    4

dev_close  move.w  mm_rechp,a2         Reclaim heap space
           jsr     (a2)                And do it
           moveq   #0,d0               CLOSE never fails
           rts                         And exit

dev_open   move.w  io_name,a2          Utility to check device names
           jsr     (a2)                And do it

           rts                         Oops, not found error return
           rts                         Oops, bad parameter error return
           bra.s   dev_ok              Yippee !

           dc.w    10                  Size of device name
           dc.b    'BLACK_HOLE'        Device name
           dc.w    0                   Has no parameters

dev_ok     move.w  mm_alchp,a2         Claim channel definition block
           moveq   #$18,d1             24 bytes is mimimum
           jsr     (a2)                Go get the space (address in A0)
           rts                         Return any error codes

dev_io     cmpi.b  #io_sbyte,d0        Check for input operation
           bge.s   not_input           It is not input

error_ef   moveq   #err_ef,d0          Return EOF on all input operations
           rts

error_bp   moveq   #err_bp,d0          BAD PARAMETER for invalid operations
           rts

not_input  cmpi.b  #io_sbyte,d0        Send a byte, do nothing
           beq.s   exit_ok

           cmpi.b  #io_sstrg,d0        Send a string ?
           beq.s   sstrg_save
           cmpi.b  #fs_save,d0         And save a file require extra code
           bne.s   not_save

sstrg_save adda.w  d2,a1               Adjust the buffer pointer
           move.w  d2,d1               And set the number of bytes sent

exit_ok    moveq   #0,d0               No errors
           rts                         And exit

not_save   cmpi.b  #fs_mdinf,d0        Read media information, EOF
           beq.s   error_ef
           cmpi.b  #fs_headr,d0        Read file header, EOF
           beq.s   error_ef
           cmpi.b  #fs_load,d0         Load a file, EOF
           beq.s   error_ef
           cmpi.b  #fs_check,d0        Silly codes such as CLS, PAPER etc
           bcs.s   error_bp            Silly ...
           cmpi.b  #fs_posre,d0
           bhi.s   error_bp            Silly ...

rest_ok    bra.s   exit_ok             All the rest,

           END
